<!-- toc -->

# 1.简介

## 1.1.适用范围

TapDB提供一套SDK，游戏开发者可以将其集成到游戏中，系统会收集玩家数据，并进行分析，最终形成数据报表，帮助游戏开发者分析玩家行为并优化游戏。

适用于Unity开发的游戏，Android支持4.0及以上的系统，iOS支持9.0及以上的系统。

## 1.2.名词解释

名词 | 含义
--- | ---
玩家 | 对应一个玩家账户，需要一个唯一的标识符。玩家是统计系统的数据统计基本单位
设备 | 安装了对应游戏的设备
付费 | 玩家使用真实货币换取游戏虚拟币或游戏道具
分包渠道 | 标识游戏安装包渠道来源，需要在代码中设置

# 2.接入方式

## 2.1.申请应用

在TapDB控制台中注册一个游戏，获得游戏对应的APP ID，这是一个16位的字符串，iOS和Android可共用一个APP ID。

## 2.2.向工程中导入SDK

在TapDB网站上下载最新的SDK，其中包含一个 TapDB.unitypackage 文件，在 Unity3D 编译器中选择 Assets --> Import Package --> Custom Package 找到 TapDB.unitypackage 文件，点击"打开按钮"即可导入成功。其中demo.cs仅是示例代码，不是SDK所需的代码。


## 2.3 Android 

### 2.3.1 权限
SDK为中需要下列权限。

权限 | 是否必须 | 用途 
--- | --- | ---
android.permission.INTERNET | 是 | 使用网络的权限 
android.permission.ACCESSNETWORKSTATE | 是 | 获取手机网络连接状态
android.permission.ACCESS_WIFI_STATE | 是 | WIFI

可选权限，强烈建议添加。sdk获取IMEI时会需要此权限;获取不到不影响功能正常使用，IMEI用于辅助数据分析，使统计结果更加精确

权限 | 是否必须 | 用途 
--- | --- | ---
android.permission.READ_PHONE_STATE | 否 | 获取手机状态信息 

### 2.3.2 第三方设备ID说明
**说明：设备ID会使数据统计更加精确，建议添加，TapDB SDK支持获取OAID的设备ID以及数美设备ID(需要手动添加oaid的sdk以及数美sdk)，获取到的设备id,会上报到服务器，辅助数据分析，使统计结果更加精准，如果不需要可以跳过该步骤**
### OAID sdk获取设备ID

TapDB SDK当前支持oaid sdk 1.0.5~1.0.23的版本，当应用集成进去之后，会自动使用oaid从而获取设备id，可以自行从OAID官网下载，也可以从此处下载[1.0.23](https://res.xdcdn.net/tapdb/Android/oaid/oaid_sdk_1.0.23.aar)的版本【如需其它版本请自行去oaid官网下载；然后将oaid sdk（下载下来是个aar)， 导入应用的工程目录


### 数美sdk获取设备ID
下载[数美sdk](https://res.xdcdn.net/tapdb/Android/xdwl/xdwl-pri1-release.aar)导入应用的工程目录

**以下均是数美sdk要求的，详情可以参照[数美sdk官网Android部分](https://help.ishumei.com/docs/tw/sdk/Android)**

**在AndroidManifest.xml添加如下权限**

```
<!-- 必选权限 -->
<uses-permission android:name="android.permission.INTERNET" />
<uses-permission android:name="android.permission.READ_PHONE_STATE" />
<uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
<uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>
<uses-permission android:name="android.permission.READ_EXTERNAL_STORAGE"/>
<uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
<!-- 强烈建议权限 -->
<uses-permission android:name="android.permission.ACCESS_COARSE_LOCATION" />
```

**注意：
SDK 默认使用 HTTP 传输数据，在 targetSdkVersion >= 28 时需要在 AndroidManifest.xml 增加如下配置**
```
<application
    ...
    android:usesCleartextTraffic="true"
    ...
   />
   ```



## 2.5 iOS 

**在 Build Setting 中的 Other Link Flag 中添加 -ObjC**


### 2.5.1 权限

**TapDB 默认不收集广告标识符（IDFA），若需要收集，可调用以下接口开启收集。请在初始化之前调用**

	TapDB.enableAdvertiserIDCollection(true);
	

 **iOS 14.5开始，获取IDFA需要配置单独权限声明**，在 info.plist 中配置 NSUserTrackingUsageDescription 及描述文案。如：请允许xxx获取并使用您的IDFA,来为您提供更好的服务。
 
 **TapDB版本 2.2.3及以上，请使用Xcode12.3或更高**。
 
 
 权限声明 | 含义
--- | ---
NSUserTrackingUsageDescription | 用来获取设备广告标识，跟踪设备




需要为iOS导出的Xcode工程引入下列依赖的框架或库

### 2.5.2 系统依赖


需要为Xcode工程引入下列依赖的框架或库

名词 | 含义 | 备注
--- | --- | ---
AdSupport.framework | 用来获取设备广告标识，跟踪设备 |  
AdService.framework | 广告框架 | optional
AppTrackingTransparency.framework | iOS14新增app追踪框架 | optional
SystemConfiguration.framework | | 
CoreMotion.framework | | 
CoreTelephony.framework | | 
Security.framework | 用来进行更好的持久化存储| 
libc++.tdb | c++ | 
libresolv.tbd | | 
libz.tbd |  | 
libsqlite3.0.tbd |  | 

### 2.5.3 第三方设备ID
TapDB 使用了第三方平台（数美）工具获取设备ID，以达到更为准确的数据归因和服务。开发者可选择是否接入该工具。若需要，则[下载](https://res.xdcdn.net/tapdb/iOS/xdwl/XdwlAntiFraud.zip)以后在工程中引入。

## 2.6.调用统计接口

在需要调用统计接口的代码中引入类TapDB，并按照后面的接口介绍调用统计接口。

# 3.接口说明

## 3.1.初始化

初始化统计系统SDK，调用这个接口是使用其它接口的先决条件，需要尽早调用。一般建议在Unity里的Start()里面调用。

```
public static void onStart(string appId, string channel, string gameVersion);
public static void onStartWithProperties(string appId, string channel, string gameVersion,Dictionary<string, object> properties)
```

字段 | 可为空 | 说明
--- | --- | ---
appId | 否 | 注册游戏时获得的APP ID
channel | 是 | 分包渠道，1.2.名词解释中有介绍
gameVersion | 是 | 游戏版本，为空时，自动获取游戏安装包的版本（Android是AndroidManifest.xml中的versionName，iOS是Xcode配置中的Version）
properties | 否 | 自定义属性，随初始化上传


## 3.2 记录一个玩家

### 3.2.1 登录
记录一个玩家（注意是平台用户，不是游戏角色！！！），当玩家登录时调用，如果是试玩用户，userId由游戏自己生成，但需要保证唯一性。

```
public static void setUser(string userId)
public static void setUserWithProperties(string userId,Dictionary<string, object> properties)
public static void setName(string name)

```

字段 | 可为空 | 说明
--- | --- | ---
userId | 否 | 玩家ID（注意是平台用户ID，不是游戏角色ID！！！），不同玩家要是唯一的，不同用户平台可能存在相同的用户ID，需要想办法做区分
userName | 是 | 玩家名称
properties | 否 | 自定义属性

### 3.2.2 登出
**玩家退出登录时，需要清除用户数据**

```
public static void clearUser() 

```


## 3.3 玩家等级

设置玩家等级，玩家登陆时或升级时调用。

```
public static void setLevel(int level)
```

字段 | 可为空 | 说明
--- | --- | ---
level | 否 | 玩家等级

## 3.4 玩家区服

设置玩家区服，玩家登陆时或更换区服时调用。

```
public static void setServer(string server)
```

字段 | 可为空 | 说明
--- | --- | ---
server | 否 | 玩家服务器

## 3.5 充值

<div style="font-size: 18px;font-weight: 500;position: relative;"><p style="position: absolute;top: -40px;left: 150px;">(<span style="color:#080">推荐使用服务端充值统计接口</span>)</p></div>

充值成功时调用

<p>提醒：<span style="color:#FA6456">由于客户端行为，不可避免会有投机者尝试破解充值；
如果没有通过服务器校验，一定会造成数据不准确，强烈建议使用服务器接口进行充值数据回调。</span>
(<a href="#41%E5%85%85%E5%80%BC%E7%BB%9F%E8%AE%A1%E6%8E%A5%E5%8F%A3">4.1.充值统计接口</a>)</p>

```
public static void onChargeSuccess(string orderId, string product, Int32 amount, string currencyType, string payment)
```

字段 | 可为空 | 说明
--- | --- | ---
orderId | 否 | 订单ID
product | 是 | 产品名称
amount | 否 | 充值金额（单位分，即无论什么币种，都需要乘以100）
currencyType | 是 | 货币类型，参考：人民币 CNY，美元 USD；欧元 EUR
payment | 是 | 支付方式，如：支付宝


## 3.7.自定义事件

需要发送自定义事件时调用，自定义事件的eventName和properties属性都必须在控制后台预先配置，等待1分钟左右的生效时间后，才可以使用SDK进行发送

```
public static void trackEvent(string eventName, Dictionary<string, object> properties)
```

字段 | 可为空 | 说明
--- | --- | ---
eventName | 否 | 事件code，需要在控制后台预先配置
properties | 是 | 事件属性，具体字段需要在控制后台预先配置

## 3.8 事件主体操作（用户、设备）

TapDB的事件主体分为用户和设备，支持对主体的某个属性做相关操作。

```
/// 初始化设备属性操作
/// @param properties 属性字典
public static void deviceInitialize(Dictionary<string, object> properties)

/// 更新设备属性操作
/// @param properties 属性字典
public static void deviceUpdate(Dictionary<string, object> properties)

/// 设备属性增加操作
/// @param properties 属性字典 暂时只支持数字属性
public static void deviceAdd(Dictionary<string, object> properties)

/// 初始化用户属性操作
/// @param properties 属性字典
public static void userInitialize(Dictionary<string, object> properties)
	
	/// 更新用户属性操作
/// @param properties 属性字典
public static void userUpdate(Dictionary<string, object> properties)
	
	/// 用户属性增加操作
/// @param properties 属性字典 暂时只支持数字属性
public static void userAdd(Dictionary<string, object> properties)
```

## 3.9 自定义属性
用户可以注册自定义属性，包括静态固定属性和动态计算属性，附带在TapDB事件中上传。


```

/// 添加静态事件属性，每个事件都将会发送
/// @param staticProperties 属性字典
public static void registerStaticProperties(Dictionary<string, object> properties)
	
/// 删除添加的静态事件属性
public static void clearStaticProperties()
	
/// 删除添加的某个静态事件属性
/// @param propertyName 属性Key
public static void unregisterStaticProperty(string propertyName)

```

# 4.服务端统计接口

## 4.1.充值统计接口

由于客户端接入充值统计可能会不太准确，这里提供服务端充值统计方法，需要忽略掉SDK中的相关充值统计接口

```
接口：https://e.tapdb.net/event
内容（注意后面还需要处理一下）：
{
    "module": "GameAnalysis", //固定
    "ip": "8.8.8.8", //充值用户的IP，可选
    "name": "charge", //固定
    "index": "APPID", //APPID注意替换成TapDB的appid
    "identify": "user_id", //user_id，必须和客户端的setUser接口传递的user_id一样，并且该用户已经通过SDK接口进行过统计
    "properties": {
        "order_id": "100000", //order_id，可选
        "amount": 100, //充值金额（单位分，即无论什么币种，都需要乘以100），必传
        "virtual_currency_amount": 100, //获赠虚拟币数量，必传，可为0
        "currency_type": "CNY", //货币类型，可选，不传或者不是正确的货币类型，统一处理成人民币分
        "product": "item1", //充值包名称，可选
        "payment": "alipay" //充值途径，可选
    }
}

假如游戏的appid为abcd1234，构建出json字符串后，需要去掉空格和换行符，然后再进行一次urlencode，再把结果作为post数据发过来
先替换换行符和空格，变成：
{"module":"GameAnalysis","name":"charge","index":"abcd1234","identify":"user_id","properties":{"order_id":"100000","amount":100,"virtual_currency_amount":100,"currency_type":"CNY","product":"item1","payment":"alipay"}}
然后urlencode，变成如下形式，某些版本的urlencode可能会把':'和','进行编码，不影响实际使用。
%7B%22module%22:%22GameAnalysis%22,%22name%22:%22charge%22,%22index%22:%22abcd1234%22,%22identify%22:%22user_id%22,%22properties%22:%7B%22order_id%22:%22100000%22,%22amount%22:100,%22virtual_currency_amount%22:100,%22currency_type%22:%22CNY%22,%22product%22:%22item1%22,%22payment%22:%22alipay%22%7D%7D
```

货币类型的格式参考<a target="_blank" href="../../docs.html#book/features/exchangeRate.html">汇率表</a>

成功判断：返回的HTTP Code为200认为发送成功，否则认为失败

## 4.2.在线数据统计接口

由于SDK无法进行准确的在线数据统计，这里提供服务端在线数据统计接口。游戏服务端可以每隔5分钟自行统计在线人数，通过接口发送到TapDB，TapDB进行数据汇总和展现。

```
接口：https://se.tapdb.net/tapdb/online
方法：POST
格式：json
必须头信息：Content-Type: application/json
```

请求内容：

参数名 | 参数类型 | 参数说明
--- | --- | ---
appid | string | TapDB的appid
onlines | array | 多条在线数据（最多100条）

其中onlines数组的结构为

参数名 | 参数类型 | 参数说明
--- | --- | ---
server | string | 服务器，TapDB对同一服务器每一个自然5分钟仅接受一次数据
online | int | 在线人数
timestamp | long | 当前统计数据的时间戳(秒)，TapDB会按照自然5分钟进行数据对齐

示例：

```
{
  "appid":"gkjasd13bbsa1sdk",
  "onlines":[{
    "server":"s1",
    "online":123,
    "timestamp":1489739590
  },{
    "server":"s2",
    "online":188,
    "timestamp":1489739560
  }]
}
```

成功判断：返回的HTTP Code为200认为发送成功，否则认为失败

___